@@ -13,7 +13,7 @@ Version 0.4402 (with libmemcached-0.44 embedded)
 
 =cut
 
-our $VERSION = '0.4402';
+our $VERSION = '0.4403';
 
 use Carp;
 use base qw(Exporter);
@@ -208,17 +208,23 @@ See L<Memcached::libmemcached::memcached_servers>.
 
 =head3 memcached_server_add
 
+=head3 memcached_server_add_with_weight
+
   memcached_server_add($memc, $hostname, $port);
+  memcached_server_add_with_weight($memc, $hostname, $port, $weight);
 
 Adds details of a single memcached server (accessed via TCP/IP) to $memc.
-See L<Memcached::libmemcached::memcached_servers>.
+See L<Memcached::libmemcached::memcached_servers>. The default weight is 0.
 
 =head3 memcached_server_add_unix_socket
 
+=head3 memcached_server_add_unix_socket_with_weight
+
   memcached_server_add_unix_socket($memc, $socket_path);
+  memcached_server_add_unix_socket_with_weight($memc, $socket_path);
 
 Adds details of a single memcached server (accessed via a UNIX domain socket) to $memc.
-See L<Memcached::libmemcached::memcached_servers>.
+See L<Memcached::libmemcached::memcached_servers>. The default weight is 0.
 
 =head3 memcached_behavior_set
 
@@ -345,21 +351,62 @@ Usually you'd just use the L</mget_into_hashref> method instead.
 
 =head2 Functions for Incrementing and Decrementing Values
 
+memcached servers have the ability to increment and decrement unsigned integer keys
+(overflow and underflow are not detected). This gives you the ability to use
+memcached to generate shared sequences of values.  
+
+See L<Memcached::libmemcached::memcached_auto>.
+
 =head3 memcached_increment
 
   memcached_increment( $key, $offset, $new_value_out );
 
-Increments the integer value associated with $key by $offset and returns the new value in $new_value_out.
-See also L<Memcached::libmemcached::memcached_auto>.
+Increments the integer value associated with $key by $offset and returns the
+new value in $new_value_out.
 
 =head3 memcached_decrement 
 
   memcached_decrement( $key, $offset, $new_value_out );
 
-Decrements the integer value associated with $key by $offset and returns the new value in $new_value_out.
-See also L<Memcached::libmemcached::memcached_auto>.
+Decrements the integer value associated with $key by $offset and returns the
+new value in $new_value_out.
 
-=cut
+=head3 memcached_increment_with_initial
+
+  memcached_increment_with_initial( $key, $offset, $initial, $expiration, $new_value_out );
+
+Increments the integer value associated with $key by $offset and returns the
+new value in $new_value_out.
+
+If the object specified by key does not exist, one of two things may happen:
+If the expiration value is MEMCACHED_EXPIRATION_NOT_ADD, the operation will fail.
+For all other expiration values, the operation will succeed by seeding the
+value for that key with a initial value to expire with the provided expiration time.
+The flags will be set to zero.
+
+=head3 memcached_decrement_with_initial
+
+  memcached_decrement_with_initial( $key, $offset, $initial, $expiration, $new_value_out );
+
+Decrements the integer value associated with $key by $offset and returns the
+new value in $new_value_out.
+
+If the object specified by key does not exist, one of two things may happen:
+If the expiration value is MEMCACHED_EXPIRATION_NOT_ADD, the operation will fail.
+For all other expiration values, the operation will succeed by seeding the
+value for that key with a initial value to expire with the provided expiration time.
+The flags will be set to zero.
+
+=head3 memcached_increment_by_key
+
+=head3 memcached_decrement_by_key
+
+=head3 memcached_increment_with_initial_by_key
+
+=head3 memcached_decrement_with_initial_by_key
+
+These are the master key equivalents of the above. They all take an extra
+initial $master_key parameter.
 
 
 =head2 Functions for Deleting Values from memcached
@@ -379,9 +426,9 @@ memcached after that many seconds.
 
 =head2 Functions for Accessing Statistics from memcached
 
-Not yet implemented.
+Not yet implemented. See L<Memcached::libmemcached::memcached_stats>.
 
-See L<Memcached::libmemcached::memcached_stats>.
+See L<walk_stats>.
 
 =cut
 
@@ -400,7 +447,7 @@ version (version of the client library, not server).
   $version = memcached_version($memc)
   ($version1, $version2, $version3) = memcached_version($memc)
 
-Returns the version of all memcached servers to respond to the version request.
+Returns the I<lowest> version of all the memcached servers.
 
 In scalar context returns a simple version string, like "1.2.3".
 In list context returns the individual version component numbers.
@@ -409,6 +456,29 @@ Returns an empty list if there was an error.
 Note that the return value differs from that of the underlying libmemcached
 library memcached_version() function.
 
+=cut
+
+sub memcached_version {
+    my $self = shift;
+
+    my @versions;
+    # XXX should be rewritten to use underlying memcached_version then
+    # return the lowest cached version from the server structures
+    $self->walk_stats('', sub {
+        my ($key, $value, $hostport) = @_;
+        push @versions, [ split /\./, $value ] if $key eq 'version';
+        return;
+    });
+
+    my $lowest = (sort {
+        $a->[0] <=> $b->[0] or $a->[1] <=> $b->[1] or $a->[2] <=> $b->[2]
+    } @versions)[0];
+
+    return join '.', @$lowest unless wantarray;
+    return @$lowest;
+}
+
+
 =head2 memcached_verbosity
 
   memcached_verbosity($memc, $verbosity)
@@ -444,12 +514,6 @@ a I<dualvar> that already contains the error string.
 
 =cut
 
-=head2 Unsupported Functions
-
-=head3 (stats)   
-
-=cut
-
 =head2 Grouping Keys On Servers
 
 Normally libmemcached hashes the $key value to select which memcached server to
@@ -618,17 +682,26 @@ passes C<$_> as the forth argument. That will work so long as the code in the
 callback doesn't alter C<$_>. If your callback code requires $stats_args you
 should change it to be a closure instead.
 
+=head2 trace_level
+
+    $memc->trace_level($trace_level);
+    $trace_level = $memc->trace_level;
+
+Sets the trace level (see L</Tracing Execution>). Returns the previous trace level.
+
 =head1 EXTRA INFORMATION
 
 =head2 Tracing Execution
 
-The C<PERL_LIBMEMCACHED_TRACE> environment variable can be used to control
-tracing. The value is read when L<memcached_create> is called.
+    $memc->trace_level($trace_level);
 
 If set >= 1 then any non-success memcached_return value will be logged via warn().
 
 If set >= 2 or more then some data types will list conversions of input and output values for function calls.
 
+The C<PERL_LIBMEMCACHED_TRACE> environment variable provides a default.
+The value is read when L<memcached_create> is called.
+
 =head2 Type Mapping
 
 For pointer arguments, undef is mapped to null on input and null is mapped to
@@ -662,7 +735,7 @@ L<http://www.tim.bunce.name>
 
 =head1 CURRENT MAINTAINER
 
-Daisuke Maki C<< <daisuke@endeworks.jp> >>
+Daisuke Maki C<< <daisuke@endeworks.jp> >> with occasional bursts of input from Tim Bunce.
 
 =head1 ACKNOWLEDGEMENTS
 
